==> vinli-android-sdk-3.0.27-sources.jar: li/vin/my/apikit/network/BaseService.kt <==
package li.vin.my.apikit.network

import com.google.gson.Gson
import li.vin.my.apikit.ApiKit
import li.vin.my.apikit.AuthSettings
import li.vin.my.apikit.errors.ConflictException
import li.vin.my.apikit.errors.DataNotFoundException
import li.vin.my.apikit.errors.GenericError
import li.vin.my.apikit.errors.GenericErrorException
import okhttp3.Cache
import okhttp3.Interceptor
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.converter.scalars.ScalarsConverterFactory

//import retrofit2.converter.scalars.ScalarsConverterFactory

/**
 * 10/31/18.
 */

abstract class BaseService<Pojo> : Queueable, Poolable {

    var call: Call<Pojo>? = null

    protected abstract fun createCall()

    var responseHandler: BaseResponseHandler? = null

    private var serviceQueue: ServiceQueue? = null
    private var queuePosition: Int = 0
    private var result: Pojo? = null
    private var throwable: Throwable? = null
    override var isCanceled: Boolean = false
    var successful: Boolean = false

    fun requestData(responseHandler: ResponseHandler<Pojo>) {
        this.responseHandler = responseHandler
        createCall()
        if (!isCanceled) {
            ServicePool.enter(this)
        }
    }

    fun getDeviceId(): String {
        if (ApiKit.instance.isDebug) {
            return "ca0a6ec9-8e42-4d7e-90d8-57dd135c5e40"
        } else {
            return AuthSettings.instance.getDeviceId()
        }
    }

    fun getVehicleId(): String {
        if (ApiKit.instance.isDebug) {
            return "1c8cd539-91fd-4aae-966d-5cf11b944fc7"
        } else {
            return AuthSettings.instance.getVehicleId()
        }
    }

    fun getCookies(): String {
        if (ApiKit.instance.isDebug) {
            return "session=Fe26.2**1e75658db601161fabff6e090eccb69df9d0ed9ab081789cc02965440bc7ba7c*PP2yeR59Wv_41ykKh02f2Q*rvUk5MIIJa5pmNbGC46ukq0mTt6x1ceGpX0kSIotZvtVwjfvqmjky4aLGSjgc3_s**2caea194cc47fe8e0ae303a6b556f2d0e51e7155a08a8585fcdfc777b5e07f12*BTwk2RsqHEZfhHr_m36CmKHvnkXiasp922KjXBs25yY"
        } else {
            return AuthSettings.instance.getSessionId()
        }
    }

    fun getAuthToken(): String {
        if (ApiKit.instance.isDebug) {
            return "Bearer 3nnVgoPsxFVNo4jvgbj8zpexFvVfsMZ58mCpMg2HsAVLmAc1dt_FIL_ZBPRG07fr"
        } else {
            return AuthSettings.instance.getAuthToken()
        }
    }

    private fun startService() {
        call?.clone()?.enqueue(object : Callback<Pojo> {
            override fun onResponse(call: Call<Pojo>, response: Response<Pojo>) {
                if (response.isSuccessful) {
                    successful = true
                    result = response.body()
                } else {
                    if (response.code() == 409) {
                        successful = false
                        throwable = ConflictException()
                    } else {
                        if (response.body() == null || (response.body() is List<*> && (response.body() as List<*>).size == 0)) {
                            successful = false
                            throwable = DataNotFoundException()
                        } else if (response.code() != 200) {
                            throwable = GenericErrorException(getErrorMessage(response))
                            successful = false
                        }
                    }
                }
                ServicePool.leave(this@BaseService)
                endService()
            }

            override fun onFailure(call: Call<Pojo>, t: Throwable) {
                successful = false
                throwable = Exception(t)
                ServicePool.leave(this@BaseService)
                endService()
            }
        })
    }

    @Suppress("UNCHECKED_CAST")
    fun endService() {
        if (serviceQueue != null) {
            if (successful) {
                serviceQueue?.serviceCompleted(result as Any, queuePosition)
            } else {
                throwable?.let {
                    serviceQueue?.serviceError(it)
                }
            }
            return
        }
        if (successful) {
            (responseHandler as ResponseHandler<Pojo>).onSuccess(result)
        } else {
            throwable?.let {
                (responseHandler as ResponseHandler<Pojo>).onError(it)
            }
        }
    }

    fun getErrorMessage(response: Response<Pojo>): String {
        var errorMessage = ""
        if (response.errorBody() != null) {
            val gson = Gson()
            val genericError = gson.fromJson(response.errorBody()?.string(), GenericError::class.java)
            errorMessage = genericError.message
        }
        return if (errorMessage.isEmpty()) {
            GenericError.getErrorMessage(response.code())
        } else {
            errorMessage
        }
    }

    fun <T> createService(serviceClass: Class<T>, baseUrl: String): T {
        return Retrofit.Builder()
            .baseUrl(baseUrl)
            .addConverterFactory(ScalarsConverterFactory.create())
            .addConverterFactory(GsonConverterFactory.create(GsonConverter.gsonConverter))
            .client(ApiKit.okHttpClient)
            .build()
            .create(serviceClass)
    }

    fun <T> createService(serviceClass: Class<T>, baseUrl: String, authToken: String): T {
        val okHttpClientBuilder = OkHttpClient.Builder()

        if (AuthSettings.instance.getAuthToken() != "Bearer" || ApiKit.instance.isDebug) {
            okHttpClientBuilder.addInterceptor(Interceptor { chain ->
                val original = chain.request()
                val requestBuilder = original.newBuilder()
                    .header("Authorization", authToken)
                    .method(original.method(), original.body())
                val request = requestBuilder.build()
                chain.proceed(request)
            })
        }
        val cacheSize: Long = 10 * 1024 * 1024 // 10 MB
        val cacheDir = Cache(ApiKit.instance.cacheDir, cacheSize)
        okHttpClientBuilder.cache(cacheDir)
        val logging = HttpLoggingInterceptor()
        logging.level = if (ApiKit.debugLogging) HttpLoggingInterceptor.Level.BODY else HttpLoggingInterceptor.Level.BASIC
        okHttpClientBuilder.addInterceptor(logging)
        okHttpClientBuilder.addInterceptor(ReceivedCookiesInterceptor())
        return Retrofit.Builder()
            .baseUrl(baseUrl)
            .addConverterFactory(GsonConverterFactory.create(GsonConverter.gsonConverter))
            .client(okHttpClientBuilder.build())
            .build()
            .create(serviceClass)

    }


    override fun startServiceInQueue(serviceQueue: ServiceQueue, queuePosition: Int) {
        this.serviceQueue = serviceQueue
        this.queuePosition = queuePosition
        createCall()
        if (!isCanceled) {
            ServicePool.enter(this)
        }
    }

    override fun setBaseResponseHandler(responseHandler: BaseResponseHandler) {
        this.responseHandler = responseHandler
    }

    override fun execute() {
        startService()
    }

    @Suppress("UNCHECKED_CAST")
    override fun completed(poolable: Poolable) {
        if (poolable is BaseService<*>) {
            val duplicate = poolable.result as Pojo
            duplicate?.let {
                successful = true
                result = duplicate
                endService()
                return
            }
            poolable.throwable.let {
                successful = false
                throwable = it
                endService()
            }
        }
    }

    override val poolId: String
        get() {
            return call?.request()?.url().toString()
        }
}

fun <Object> BaseService<Object>.setCanceled(canceled: Boolean) {
    isCanceled = canceled
}

==> vinli-android-sdk-3.0.31-sources.jar: li/vin/my/apikit/network/BaseService.kt <==
package li.vin.my.apikit.network

import com.google.gson.Gson
import li.vin.my.apikit.ApiKit
import li.vin.my.apikit.AuthSettings
import li.vin.my.apikit.errors.ConflictException
import li.vin.my.apikit.errors.DataNotFoundException
import li.vin.my.apikit.errors.GenericError
import li.vin.my.apikit.errors.GenericErrorException
import okhttp3.Cache
import okhttp3.Interceptor
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.converter.scalars.ScalarsConverterFactory
import java.util.concurrent.TimeUnit

//import retrofit2.converter.scalars.ScalarsConverterFactory

/**
 * 10/31/18.
 */

abstract class BaseService<Pojo> : Queueable, Poolable {

    var call: Call<Pojo>? = null

    protected abstract fun createCall()

    var responseHandler: BaseResponseHandler? = null

    private var serviceQueue: ServiceQueue? = null
    private var queuePosition: Int = 0
    private var result: Pojo? = null
    private var throwable: Throwable? = null
    override var isCanceled: Boolean = false
    var successful: Boolean = false

    fun requestData(responseHandler: ResponseHandler<Pojo>) {
        this.responseHandler = responseHandler
        createCall()
        if (!isCanceled) {
            ServicePool.enter(this)
        }
    }

    fun getDeviceId(): String {
        if (ApiKit.instance.isDebug) {
            return "ca0a6ec9-8e42-4d7e-90d8-57dd135c5e40"
        } else {
            return AuthSettings.instance.getDeviceId()
        }
    }

    fun getVehicleId(): String {
        if (ApiKit.instance.isDebug) {
            return "1c8cd539-91fd-4aae-966d-5cf11b944fc7"
        } else {
            return AuthSettings.instance.getVehicleId()
        }
    }

    fun getCookies(): String {
        if (ApiKit.instance.isDebug) {
            return "session=Fe26.2**1e75658db601161fabff6e090eccb69df9d0ed9ab081789cc02965440bc7ba7c*PP2yeR59Wv_41ykKh02f2Q*rvUk5MIIJa5pmNbGC46ukq0mTt6x1ceGpX0kSIotZvtVwjfvqmjky4aLGSjgc3_s**2caea194cc47fe8e0ae303a6b556f2d0e51e7155a08a8585fcdfc777b5e07f12*BTwk2RsqHEZfhHr_m36CmKHvnkXiasp922KjXBs25yY"
        } else {
            return AuthSettings.instance.getSessionId()
        }
    }

    fun getAuthToken(): String {
        if (ApiKit.instance.isDebug) {
            return "Bearer 3nnVgoPsxFVNo4jvgbj8zpexFvVfsMZ58mCpMg2HsAVLmAc1dt_FIL_ZBPRG07fr"
        } else {
            return AuthSettings.instance.getAuthToken()
        }
    }

    private fun startService() {
        call?.clone()?.enqueue(object : Callback<Pojo> {
            override fun onResponse(call: Call<Pojo>, response: Response<Pojo>) {
                if (response.isSuccessful) {
                    successful = true
                    result = response.body()
                } else {
                    if (response.code() == 409) {
                        successful = false
                        throwable = ConflictException()
                    } else {
                        if (response.body() == null || (response.body() is List<*> && (response.body() as List<*>).size == 0)) {
                            successful = false
                            throwable = DataNotFoundException()
                        } else if (response.code() != 200) {
                            throwable = GenericErrorException(getErrorMessage(response))
                            successful = false
                        }
                    }
                }
                ServicePool.leave(this@BaseService)
                endService()
            }

            override fun onFailure(call: Call<Pojo>, t: Throwable) {
                successful = false
                throwable = Exception(t)
                ServicePool.leave(this@BaseService)
                endService()
            }
        })
    }

    @Suppress("UNCHECKED_CAST")
    fun endService() {
        if (serviceQueue != null) {
            if (successful) {
                serviceQueue?.serviceCompleted(result as Any, queuePosition)
            } else {
                throwable?.let {
                    serviceQueue?.serviceError(it)
                }
            }
            return
        }
        if (successful) {
            (responseHandler as ResponseHandler<Pojo>).onSuccess(result)
        } else {
            throwable?.let {
                (responseHandler as ResponseHandler<Pojo>).onError(it)
            }
        }
    }

    fun getErrorMessage(response: Response<Pojo>): String {
        var errorMessage = ""
        if (response.errorBody() != null) {
            val gson = Gson()
            val genericError = gson.fromJson(response.errorBody()?.string(), GenericError::class.java)
            errorMessage = genericError.message
        }
        return if (errorMessage.isEmpty()) {
            GenericError.getErrorMessage(response.code())
        } else {
            errorMessage
        }
    }

    fun <T> createService(serviceClass: Class<T>, baseUrl: String): T {
        return Retrofit.Builder()
            .baseUrl(baseUrl)
            .addConverterFactory(ScalarsConverterFactory.create())
            .addConverterFactory(GsonConverterFactory.create(GsonConverter.gsonConverter))
            .client(ApiKit.okHttpClient)
            .build()
            .create(serviceClass)
    }

    fun <T> createService(serviceClass: Class<T>, baseUrl: String, authToken: String): T {
        val okHttpClientBuilder = OkHttpClient.Builder()

        if (AuthSettings.instance.getAuthToken() != "Bearer" || ApiKit.instance.isDebug) {
            okHttpClientBuilder.addInterceptor(Interceptor { chain ->
                val original = chain.request()
                val requestBuilder = original.newBuilder()
                    .header("Authorization", authToken)
                    .method(original.method(), original.body())
                val request = requestBuilder.build()
                chain.proceed(request)
            })
        }
        val cacheSize: Long = 10 * 1024 * 1024 // 10 MB
        val cacheDir = Cache(ApiKit.instance.cacheDir, cacheSize)
        okHttpClientBuilder.cache(cacheDir)
        val logging = HttpLoggingInterceptor()
        logging.level = if (ApiKit.debugLogging) HttpLoggingInterceptor.Level.BODY else HttpLoggingInterceptor.Level.BASIC
        okHttpClientBuilder.addInterceptor(logging)
        okHttpClientBuilder.addInterceptor(ReceivedCookiesInterceptor())
        okHttpClientBuilder.connectTimeout(45, TimeUnit.SECONDS)
        okHttpClientBuilder.readTimeout(45, TimeUnit.SECONDS)
        return Retrofit.Builder()
            .baseUrl(baseUrl)
            .addConverterFactory(GsonConverterFactory.create(GsonConverter.gsonConverter))
            .client(okHttpClientBuilder.build())
            .build()
            .create(serviceClass)

    }


    override fun startServiceInQueue(serviceQueue: ServiceQueue, queuePosition: Int) {
        this.serviceQueue = serviceQueue
        this.queuePosition = queuePosition
        createCall()
        if (!isCanceled) {
            ServicePool.enter(this)
        }
    }

    override fun setBaseResponseHandler(responseHandler: BaseResponseHandler) {
        this.responseHandler = responseHandler
    }

    override fun execute() {
        startService()
    }

    @Suppress("UNCHECKED_CAST")
    override fun completed(poolable: Poolable) {
        if (poolable is BaseService<*>) {
            val duplicate = poolable.result as Pojo
            duplicate?.let {
                successful = true
                result = duplicate
                endService()
                return
            }
            poolable.throwable.let {
                successful = false
                throwable = it
                endService()
            }
        }
    }

    override val poolId: String
        get() {
            return call?.request()?.url().toString()
        }
}

fun <Object> BaseService<Object>.setCanceled(canceled: Boolean) {
    isCanceled = canceled
}
